#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QFile>
#include <QXmlStreamReader>
#include <QXmlStreamAttribute>

#include <QDrag>
#include <QMimeData>
#include <QIcon>
#include <QGraphicsSceneDragDropEvent>

int ParamList::number(QString key, int default_value)
{
    QString s = this->value (key);
    bool ok;
    int n = s.toInt (&ok);
    if (!ok)
        n = default_value;
    return n;
}

/* Color Button */

ColorButton::ColorButton(QString s)
  : name (s), color (QColor (s))
{
    setText (s);
    setToolTip (s);

    QPixmap p (24, 24);
    p.fill (QColor (0, 0, 0, 0));

    QPainter m (&p);
    m.setPen (color);
    m.setBrush (color);
    m.drawEllipse (2, 2, 20, 20);

    QIcon icon (p);
    setIcon (icon);
}

void ColorButton::mousePressEvent(QMouseEvent *event)
{
    QDrag * drag = new QDrag (this);
    QMimeData * data = new QMimeData ();
    data->setText (name);
    data->setColorData (color);
    drag->setMimeData(data);

    QPixmap p = this->icon().pixmap(12, 12);
    drag->setPixmap(p);

    drag->start(Qt::CopyAction | Qt::MoveAction);
};

/* Rectangle */

void Rectangle::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    event->setAccepted(true);
}

void Rectangle::dropEvent(QGraphicsSceneDragDropEvent *event)
{
   const QMimeData * data =  event->mimeData();
   if (data->hasColor())
   {
       QColor c =  data->colorData().value <QColor> ();
       if (event->proposedAction() == Qt::MoveAction) /* shift */
          setPen (c);
       else
          setBrush (c);
   }
}

Rectangle::Rectangle()
{
    setAcceptDrops(true);
}

/* Ellipse */

void Ellipse::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
{
    event->setAccepted(true);
}

void Ellipse::dropEvent(QGraphicsSceneDragDropEvent *event)
{
   const QMimeData * data =  event->mimeData();
   if (data->hasColor())
   {
       QColor c =  data->colorData().value <QColor> ();
       if (event->proposedAction() == Qt::MoveAction) /* shift */
          setPen (c);
       else
          setBrush (c);
   }
}

Ellipse::Ellipse()
{
    setAcceptDrops(true);
}

/* MainWindow */

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    QList <QString> names;
    names << "red" << "blue" << "green" << "yellow";

    for (auto name : names)
    {
       ColorButton * b = new ColorButton (name);
       ui->mainToolBar->addWidget (b);
    }

    scene = new QGraphicsScene;
    ui->graphicsView->setScene (scene);

    // scene->addLine (0, 0, 100, 100, QColor ("blue"));
    scene->setSceneRect (0, 0, 1024, 800);

    readFile ("../xml/data.xml");
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::readFile (QString fileName)
{
    QFile file (fileName);
    if (file.open (QFile::ReadOnly | QFile::Text ))
    {
        QTreeWidgetItem * branch =
                ui->treeWidget->invisibleRootItem();
        QGraphicsItem * target = NULL;

        QXmlStreamReader reader (&file);
        while (! reader.atEnd())
        {
            if (reader.isStartElement())
            {
                QString s = reader.name().toString();
                QTreeWidgetItem * item = new QTreeWidgetItem;
                item->setText (0, s);
                item->setForeground (0, QColor ("blue"));
                branch->addChild (item);
                branch = item;

                ParamList list;
                // for (QXmlStreamAttribute a : reader.attributes())
                for (auto a : reader.attributes())
                {
                   auto node = new QTreeWidgetItem;
                   QString t = a.name().toString();
                   QString v = a.value().toString();
                   node->setText (0, t + " = " + v);
                   node->setForeground (0, QColor ("green"));
                   item->addChild (node);

                   // list.insert(t, v);
                   list [t] = v;
                }

                if (s == "rectangle")
                   target = createRectangle (target, list);
                else if (s == "ellipse")
                   target = createEllipse (target, list);
            }
            else if (reader.isCharacters())
            {
                QString s = reader.text().toString().simplified();
                if (s != "")
                {
                   QTreeWidgetItem * item = new QTreeWidgetItem;
                   item->setText (0, s);
                   item->setForeground (0, QColor ("orange"));
                   branch->addChild (item);
                }
            }
            else if (reader.isEndElement())
            {
                branch = branch->parent();

                QString s = reader.name().toString();
                if (s == "rectangle" || s == "ellipse")
                   target = target->parentItem ();
            }
            reader.readNext();
        }
        ui->treeWidget->expandAll();
    }
}

Rectangle * MainWindow::createRectangle
                        (QGraphicsItem * target,
                         ParamList list)
{
    Rectangle * shape = new Rectangle;

    int w = list.number ("w", 100);
    int h = list.number ("h", 40);

    shape->setRect (0, 0, w, h);

    modifyShape (target, shape, list);

    return shape;
}

Ellipse * MainWindow::createEllipse
                      (QGraphicsItem * target,
                       ParamList list)
{
    Ellipse * shape = new Ellipse;

    int w = list.number ("w", 10);
    int h = list.number ("h", 10);

    shape->setRect (0, 0, w, h);

    modifyShape (target, shape, list);

    return shape;
}

void MainWindow::modifyShape (QGraphicsItem * target,
                              ShapeItem * shape,
                              ParamList list)
{
    int x = list.number ("x", 20);
    int y = list.number ("y", 20);
    shape->setPos (x, y);

    QString pen = list ["pen"];
    if (pen != "")
       shape->setPen (QColor (pen));

    QString brush = list ["brush"];
    if (brush != "")
       shape->setBrush (QColor (brush));

    shape->setFlag (QGraphicsItem::ItemIsMovable);

    if (target == NULL)
       scene->addItem (shape);
    else
       shape->setParentItem (target);
}

